/* Interactive (or batch) interface to magfield.h
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** 2001-01-31 Added include of <string.h> for declaration of return type
**            for strncpy() to make compiler happy. Removed unused 
**            variables: latdeg, latmin, latsec, latdec, londeg, lonmin,
**            lonsec and londec. Added declaration for parse_l() to
**            beginning to make compiler happy. Corrected fprintf()
**            format statements "%lf" to "%f". ("%lf" is a scanf()
**            specification).  Fixed string variable args to sscanf()
**            by removing incorrect leading "&" (string variables are
**            "address of" by default).
**            Jim Seymour (jseymour@LinxNet.com)
**
** 2001/02/01 void main -> int main 
**            EAW
*/ 

#include	<stdio.h>
#include	<stdlib.h>
#include        <string.h>
#include	<math.h>
#include        "magfield.h"

void outputvar(double var);
void outputfield(double *field,double var);
void setvariable(char *inputstring,
     int *pmodel,int *pmm,int *pdd,int *pyy, char *plusstr,int *poutput_field);
int readinputdata(char *inputstring,double *lat,double *lon, double *alt);
int plusstring(char *str,int *platsign,int *plonsign, int *pvarsign);
static int parse_l();

int main(){

char *def_plusstr="NWW", plusstr[4],newplusstr[4];
int latsign,lonsign,varsign;
int model=12;
double lat,lon,alt,default_alt=0,var;
int yy=15,mm=1,dd=1;  /* defaults */

const char *helpstring=
"  To calculate magnetic variation enter \"lat lon [altitude]\" at the >\n"
"prompt. The optional altitude defaults to 0 km.  Lat/lon may be\n"
"entered in [-]DD[.DD], [-]DD:MM[.MM] or [-]DD:MM:SS[.SS] formats. The\n"
"decimal portions are optional.\n"
"  The magnetic field model, the date, the sign and output defaults can\n"
"be changed with the s[et] command. \"set variable value\" sets\n"
"\"variable\" to \"value\".\n"
" The model variable can be set to IGRF?""? or WMM?""? ?""?=85/90/95/00/05/10/15\n"
"  Dates are changed with \"set date MM/DD/YY\". The models have a\n"
"nominal five year validity.  The \"plus\" variable defaults to NWW,\n"
"which means that N latitudes, W longitudes and W variations are\n"
"designated as positive. S latitudes and E longitudes must then be\n"
"entered with a - sign. It is changed by \"set plus XXX\", where X is\n"
"N,S,E or W.\n"
"  With a W positive default for variation, to get a magnetic course\n"
"from a true course you *add* the variation, with W negative, you subtract.\n"
"\"set field yes\" outputs field components in addition to variation\n"
"\"set\" gives the current defaults.\n"
"  Example commands:\n"
"\"help\"  " "\"set date 1/1/85\"  ""\"set plus NEW\" \"set field yes\"\n"
"\"set model WMM85\"  " "\"set\"  " "\"52:4:30 -121:23.5759\"\n"
"\"52.075 -121:23:34.555 2.3\"  " "\"quit\"\n";

const char welcomestring[] =
"Geomagnetic Field and Variation Calculator v1.05\nCopyright 2000 E A Williams"
" All rights reserved.\n";
const char enterstring[] = "Enter lat lon [alt], type h for help, type q to quit\n";
char inputstring[81];
double field[6];
int output_field=0;  /* =1 outputs mag field */

strncpy(plusstr,def_plusstr,4);
strncpy(newplusstr,def_plusstr,4);
plusstring(plusstr,&latsign,&lonsign,&varsign); /* initialize signs */
 fprintf(stdout,welcomestring);
fprintf(stdout,"Defaults are:\n");
setvariable("s",&model,&mm,&dd,&yy,plusstr,&output_field);  
                        /* Write out variable settings */
fprintf(stdout,"\n");
 fprintf(stdout,enterstring);



for (;;) {
  fputs("> ",stdout);
  if (fgets(inputstring,80,stdin)==NULL) return 0;  /* quit at EOF */
  switch (inputstring[0]){
  case 'q': case 'Q':
     return 0;
  case ' ': 
  case '\t':
  case '\n':
     break;
  case 's': case 'S':
     setvariable(inputstring,&model,&mm,&dd,&yy,newplusstr,&output_field);
     if (plusstring(newplusstr,&latsign,&lonsign,&varsign)){
        strncpy(plusstr,newplusstr,3); /* if good update */
      }
     else{
        strncpy(newplusstr,plusstr,3); /* if not restore */
      }
     break;
  case '0': case '1': case '2': case '3': case '4':
  case '5': case '6': case '7': case '8': case '9': case '-':
     alt=default_alt;
     if (readinputdata(inputstring,&lat,&lon,&alt)){
       var=rad_to_deg(varsign*SGMagVar(latsign*deg_to_rad(lat),lonsign*deg_to_rad(lon),alt,
                yymmdd_to_julian_days(yy,mm,dd),model,field));
       if (output_field) outputfield(field,var);
       else outputvar(var);
     }
     else{
       fputs("Couldn't parse lat/lon data\n",stdout);
     }
     break;
  case 'h': case 'H':
     fputs(helpstring,stdout);
     break;
   default:
     fputs("Unrecognized command: type ""h"" for help\n",stdout);
     break;
  }
}
 return 0;
}

void outputvar(double var){
  fprintf(stdout,"% 2.1f\n",var);
}

void outputfield(double *field,double var){
fprintf(stdout,
"Br=%6.0f, Btheta=%6.0f, Bphi=%6.0f (nTesla) (Earth centered sph. coords.)\n"
"Bx=%6.0f (N), By=%6.0f (E), Bz=%6.0f (Down) (nTesla)\n"
"Bh=%6.0f (Horiz), B=%6.0f (Total)\n"
"Dip angle=%2.1f degrees, Variation=% 2.1f degrees\n",
field[0],field[1],field[2],field[3],field[4],field[5],
pow(field[3]*field[3]+field[4]*field[4],0.5),
pow(field[3]*field[3]+field[4]*field[4]+field[5]*field[5],0.5),
rad_to_deg(atan(field[5]/pow(field[3]*field[3]+field[4]*field[4],0.5))),var);
}

void setvariable(char *inputstring,
   int *pmodel,int *pmm,int *pdd,int *pyy, char *plusstr,int *poutput_field){
/* parse input string looking for "set variable value" */

char s1[80],s2[80],s3[80];
 char *model_name[13]={"IGRF90","WMM85","WMM90","WMM95","IGRF95","WMM00","IGRF00","WMM05","IGRF05","WMM10","IGRF10","WMM15","IGRF15"};
int nitems,nmodel,change=0,mm,dd,yy;

nitems=sscanf(inputstring,"%s %s %s",s1,s2,s3);
if (nitems==0 || nitems==2){
  fprintf(stdout,"Syntax is \"set variable value\"\n");
  return;
}

if (nitems==1 ){ /* show values */
  fprintf(stdout,"model %s\ndate %2d/%2d/%2d\n",model_name[*pmodel-1],*pmm,*pdd,*pyy);
  fprintf(stdout,"plus %s (%c latitude, %c longitude, %c variation are positive)\n",
                  plusstr,plusstr[0],plusstr[1],plusstr[1]);
  fprintf(stdout,"field %s (output mag field components  yes/no)\n",
              ((*poutput_field) ?  "yes":"no"));
  return;
}

if (!strncmp(s2,"model",5)){ /* change model ? */
  for (nmodel=0;nmodel<13;nmodel++){
     if (!strncmp(s3,model_name[nmodel],6)){
          *pmodel=nmodel +1;
          change=1;
	  }
  }  
  if (change==0){  /* didn't match */
    fprintf(stdout,"Unknown model: %s\nModels are %s %s %s %s %s %s %s %s %s %s %s %s %s\n",
	    s3,model_name[0],model_name[1],model_name[2],model_name[3],model_name[4],model_name[5],model_name[6],
            model_name[7],model_name[8],model_name[9],model_name[10],model_name[11],model_name[12]);
  }
}
else if (!strncmp(s2,"date",5)){ /* change date ? */
   if(sscanf(s3,"%d/%d/%d",&mm,&dd,&yy)==3){
      *pmm=mm;
      *pdd=dd;
      *pyy=yy;  /* got date */
   }
   else{
      fprintf(stdout,"Syntax is \"set date mm/dd/yy\"\n");
      return;   /* didn't get date */
   }
 }
else if (!strncmp(s2,"plus",4)){ /* change sign defaults ? */
       strncpy(plusstr,s3,3);
}
else if (!strncmp(s2,"field",5)){
    if (s3[0]=='y'){
       *poutput_field=1;
     }
    else if (s3[0]=='n'){
       *poutput_field=0;
     }
    else{
       fprintf(stdout,"\"set field yes\" or \"set field no\"\n");
     }
}
else{
    fprintf (stdout,"Variable %s not recognized\n",s2);
    }
}

int readinputdata(char *inputstring,double *plat,double *plon, double *palt){
/* parse inputstring for lat lon [alt] */
/* for now just dd.ddd */
char latstr[80],lonstr[80],altstr[80];
int nitems;
double lat,lon,alt;

nitems=sscanf(inputstring,"%s %s %s",latstr,lonstr,altstr);

if (nitems<2){
   /* didn't find at least 2 args */
   return 0;
}
else if (nitems==2){
   if (parse_l(latstr,&lat)){
      *plat=lat;
    }
   else{
     fprintf(stdout,"Latitude format not recognized\n"
		     "DD[.DD] DD:MM[.MM] DD:MM:SS[.SS] are allowed\n");
     return 0;
   }
   if (parse_l(lonstr,&lon)){
      *plon=lon;
    }
   else{
     fprintf(stdout,"Longitude format not recognized\n");
     return 0;
   }
 }
else if (nitems==3){
      if (parse_l(latstr,&lat)){
      *plat=lat;
    }
   else{
     fprintf(stdout,"Latitude format not recognized\n"
		     "[-]DD[.DD] [-]DD:MM[.MM] [-]DD:MM:SS[.SS] are allowed\n");
     return 0;
   }
   if (parse_l(lonstr,&lon)){
      *plon=lon;
    }
   else{
     fprintf(stdout,"Longitude format not recognized\n");
     return 0;
   }
   sscanf(altstr,"%lf",&alt);
   *plat=lat;
   *plon=lon;
   *palt=alt;
}
return 1;
}

int plusstring(char *str,int *platsign,int *plonsign, int *pvarsign){
/* set signs from str */
const char str_error[] = "Syntax \"set plus ABC\" where A is N or S, B and C are E or W\n"
"e.g. \"set plus NEE\" makes N latitudes,E longitudes and E variations positive\n"
"S latitudes and W longitudes would then have to be entered with a - sign\n"
     "W variations will be output with a - sign\n"; 
if (str[0]=='N') *platsign=1;
else if (str[0]=='S') *platsign=-1;
else {fprintf(stdout,str_error);return 0;}

if (str[1]=='E') *plonsign=1;
else if (str[1]=='W') *plonsign=-1;
else  {fprintf(stdout,str_error);return 0;}

if (str[2]=='E') *pvarsign=1;
else if (str[2]=='W') *pvarsign=-1;
else {fprintf(stdout,str_error);return 0;}

return 1; /* normal exit */
}

int parse_l(char *str,double *d){
/* parse lat/lon in form DD.DDD, DD:MM.MMM or DD:MM:SS.SSS into double DD.DDD */
int ddi,mmi;
double ddd,mmd,ssd;

if (sscanf(str,"%d:%d:%lf",&ddi,&mmi,&ssd)==3){
 *d= (str[0]=='-') ? -(abs(ddi)+(mmi+ssd/60.)/60.) : (abs(ddi)+(mmi+ssd/60.)/60.);
 return 1;
 }
else if (sscanf(str,"%d:%lf",&ddi,&mmd)==2){
 *d= (str[0]=='-') ? -(abs(ddi)+mmd/60.) : (abs(ddi)+mmd/60.) ;
 return 1;
 }
else if (sscanf(str,"%lf",&ddd)==1){
 *d=ddd;
 return 1;
 }
else{
 return 0; /* format not recognised */
}
}
 
